@@ -17,7 +17,7 @@ except ImportError:
    raise
 
 try:
-    from Stampy import hashtable, genome, reader, formatter, mapper, singleendedmapper, recalibrate, mapstats, fastqformatter, utils
+    from Stampy import hashtable, genome, reader, formatter, mapper, singleendedmapper, recalibrate, mapstats, fastqformatter, utils, multithread, filez
 except ImportError:
     print >> sys.stderr,"Could not import main python modules -- please check that the Python interpreter matches the installed Stampy version"
 
@@ -25,41 +25,52 @@ import plugins
 
 from ext import makeconstants
 
-short_opts =  "?!g:G:d:v:H:h:f:i:s:R:M:T:S:P:o:f:"
+
+short_opts =  "?!g:G:d:v:H:h:f:i:s:R:M:T:S:P:A:o:f:t:"
 string_opts = ["genome=","logfile=","hash=","outputformat=","output=","inputformat=","stats=","keep-recalibrated=",
                "map=","comment=","recalibrate=","qualitybase=","recaldatasuffix=","testmap=","recaldatadir=","remote=",
-               "readgroup=","processpart=","parse=","species=","assembly=","referenceuri=","pg="]
+               "readgroup=","processpart=","parse=","species=","assembly=","referenceuri=","pg=","labelfilter=","separator1=",
+               "separator2=","bamsortprefix=","adapter-strip="]
 multi_opts = []
-float_opts =  ["recalfraction=","toppostfraction=","substitutionrate="]
+float_opts =  ["recalfraction=","substitutionrate="]
 int_opts =    ["verbosity=","bits=","maxcount=","maxscore=","minposterior=","numrecords=","lowqthreshold=","seed=",
                "insertsize=","insertsd=","maxfingerprintvariants=","linearalignmentband=","simulate-minindellen=",
                "simulate-maxindellen=","tryvariants=","fastaqual=","simulate-numsubstitutions=","gapopen=","gapextend=",
                "recalscoreprefix=","svprior=","longindelprior=","baseentropy=","banding=","xa-max=","xa-max-discordant=",
-               "insertsize2=","insertsd2="]
+               "insertsize2=","insertsd2=","padding=","maxpairseeds=","paircandlikethres=","maxbasequal=","threads=",
+               "bamsortbuflen=","bamsortmemory=","unclipq="]
 bool_opts =   ["logit","sensitive","fast","simulate-duplications","solexa","solexaold","sanger","noautosense","norefoutput",
-               "stripns","keeplabel","rightmost","alignquals","keepreforder","noparseNCBI","mdtag","ignore-improper-pairs","baq"]
+               "stripns","keeplabel","rightmost","alignquals","keepreforder","alphabetical","noparseNCBI","mdtag","strip-adapter",
+               "ignore-improper-pairs","baq","overwrite","casava8","gatkcigarworkaround","bamkeepgoodreads","xcigar","cigarx"]
 long_opts =   ["help","build-genome=","build-hash=","recalibrate=","map=","simulate="] + string_opts + int_opts + float_opts + bool_opts
 
+oldhelp = """
+ -R FILE, --recalibrate=FILE[,FILE] Compute recalibration data from fastq file(s) (not usually recommended)
+ --recalfraction=F                  Fraction of reads to use for recalibration [0.01]
+ --recaldatadir=DIR                 Directory for recalibration data [same as fastq file]
+ --recalscoreprefix=N               Read prefix length over which likelihood is calculated for recalibration [20]
+"""
+
+
 help = """%s\n\nUsage: %s [options] [files]
 
-Command options
+Command options (one required)
  -G PREFIX, --build-genome=PREFIX   Build genome index PREFIX.stidx from fasta file(s) on command line
  -H PREFIX, --build-hash=PREFIX     Build hash PREFIX.sthash
- -M FILE, --map=FILE[,FILE]         Map fastq file(s).  Use FILE.recaldata for recalibration if available
- -R FILE, --recalibrate=FILE[,FILE] Compute recalibration data from fastq file(s)
- -S FILE, --simulate=FILE           Simulate reads following empirical (recalibrated) qualities from read FILE
+ -M FILE, --map=FILE[,FILE]         Map fastq/fasta/BAM file(s)
+ -S FILE, --simulate=FILE           Simulate reads following empirical qualities from read FILE
  -P FILE, --parse=FILE              Parse simulated SAM file, and report statistics
+ -A FILE                            Convert qualities; strip adapters
 
 Mapping options:
  -g PREFIX, --genome=PREFIX         Use genome index file PREFIX.stidx
  -h PREFIX, --hash=PREFIX           Use hash file PREFIX.sthash
+ -t N, --threads=N                  Number of threads to use [1]
  --numrecords=N                     Number of records to process [all]
  --processpart=A/B                  Process the Ath read (pair) in every block of B read (pairs) [1/1]
- --recalfraction=F                  Fraction of reads to use for recalibration [0.01]
- --minposterior=N                   Minimum read mapping phred posterior [mapping: no filtering; recalibration: 20]
- --maxscore=N                       Maximum read likelihood phred score [mapping: no filtering; recalibration: 60]
- --recaldatadir=DIR                 Directory for recalibration data [same as fastq file]
- --qualitybase=N                    Character or ASCII for quality score 0 in input [33 or !]
+ --minposterior=N                   Minimum read mapping phred posterior [no filtering]
+ --maxscore=N                       Maximum read likelihood phred score [no filtering]
+ --qualitybase=C                    Character or ASCII for quality score 0 in input [33 or !]
  --logit                            Qualities are logit (log p/1-p) rather than phred (log p) scores [phred]
  --solexa, --solexaold, --sanger    Short for: --qualitybase=@; --qualitybase=@ --logit; default
  --substitutionrate=F               Set substitution rate for mapping and simulation [0.001]
@@ -69,6 +80,7 @@ Mapping options:
  --insertsd=N                       (Initial) standard deviation for insert size [60]
  --insertsize2=N                    (Initial) mean insert size for mate pairs [-2000]
  --insertsd2=N                      (Initial) standard deviation for mate pairs [-1 = deactivated]
+ --maxpairseeds=N                   Number of ambiguous single mappings to take forward for paired realignment [25]
  --noautosense                      Do not auto-sense insert size distribution(s) [False]
  --sensitive                        More sensitive; about 25-50%%%% slower
  --fast                             Less sensitive; about 50%%%% (paired-end) to 100%%%% (single-ended) faster
@@ -79,14 +91,16 @@ Output options:
  --rightmost                        Flush gaps to rightmost position in reference coordinates [leftmost]
  --xa-max                           Number of alternative hits to output for single/concordant paired reads [0, max 20]
  --xa-max-discordant                Number of alternative hits to output for discordant paired reads [0]
+ --overwrite                        Allow overwriting of output files
  --baq                              (SAM format) Compute base-alignment quality (BAQ; BQ tag)
  --alignquals                       (SAM format) Compute posterior alignment probabilities (YQ tag)
+ --xcigar                           (SAM format) Compute the extended CIGAR string (XC tag)
  --readgroup=ID:id,tag:value,...    (SAM format) Set readgroup tags (ID,SM,LB,DS,PU,PI,CN,DT,PL); ,, quotes ,
  --norefoutput                      (SAM format) Use = signs at read positions that match the reference
  --comment=comment1%%%%comment2%%%%...    (SAM format) Add @COmments.  %%%% separate lines; commas are converted into tabs
  --comment=@F                       (SAM format) Add @COmments from file F
 
-Simulation options (use with -S or -T):
+Simulation options (use with -S):
  --seed=N                           Set random number seed [1]
  --simulate-minindellen=N           Minimum indel length to simulate; negative is deletion [0]
  --simulate-maxindellen=N           Maximum indel length to simulate; negative is deletion [0]
@@ -94,20 +108,32 @@ Simulation options (use with -S or -T):
  --simulate-numsubstitutions=N      Insert N substitutions in read (pairs) (default = Poisson distribution)
 %s
 Advanced options:
- --maxcount=N                       Maximum multiplicity of repeat words (used when building hash) [200]
- --assembly=S                       Set assembly identifier (for option -G; appears in @SQ field (AS tag) in SAM output)
- --species=S                        Set species identifier (for option -G; appears in @SQ field (SP tag) in SAM output)
- --referenceuri=S                   Set URI for the reference (for option -G; appears in @SQ field (UR tag) in SAM output)
+ --maxcount=N                       (-H) Maximum multiplicity of repeat words [200]
+ --assembly=S                       (-G) Set assembly identifier (appears in @SQ field (AS tag) in SAM output)
+ --species=S                        (-G) Set species identifier (appears in @SQ field (SP tag) in SAM output)
+ --referenceuri=S                   (-G) Set URI for the reference (appears in @SQ field (UR tag) in SAM output)
+ --noparseNCBI                      (-G) Don't parse but just copy gi|nnn|ref|xxx| labels in NCBI fasta files
+ --strip-adapter                    (-A) Strip adapters from fastq files
  --inputformat=FMT                  Read input format (fasta, fastq, sam, bam) [fastq]
  --banding=N                        Band size for banded alignment; -1=none [60]
  --fastaqual=N                      Base quality for fasta input [30]
- --recalscoreprefix=N               Read prefix length over which likelihood is calculated for recalibration [20]
+ --maxbasequal=N                    Maximum accepted base quality [50]
  --svprior=N                        Prior probability of read pair bridging a SV (phred score) [55]
  --longindelprior=N                 Prior probability of read pair bridging a long indel (phred score) [40]
  --baseentropy=N                    Entropy of inserted nucleotides (phred score) [5]
- --remote=U                         Command to obtain input reads from URL %%%%s; shortcuts are 'wget' and 'scp'
- --keepreforder                     (-M) Use reference order in .fa/.stidx file, rather than alphabetical order
- --noparseNCBI                      (-G) Stop parsing gi|nnn|ref|xxx| labels in NCBI fasta files
+ --remote=U                         Command to get input reads from URL %%%%s; shortcuts are 'wget' and 'scp'
+ --keepreforder                     (-M) (default) Use reference order in .fa/.stidx file, rather than alphabetical
+ --alphabetical                     (-M) Order reference identifiers alphabetically
+ --labelfilter=R                    (-M) Only map reads / read pairs whose label match the regular expression R
+ --casava8                          (-M) Parse Casava v1.8 FastQ headers; include but do not map filtered reads
+ --separator1=c                     (-M) Use 'c' as separator for Casava data columns [:]
+ --separator2=c                     (-M) use 'c' as separator between first 7 columns, and the auxiliary columns [ ]
+ --gatkcigarworkaround              (-M) Remove adjacent I/D CIGAR operators (valid in SAM spec, but trips up GATK)
+ --unclipq                          (-M X.bam / -M --bamoptions) Undo soft clipping over low Q bases (-1=disable) [10]
+ --bamsortprefix=S                  (-M X.bam) Temporary storage for sorting [--output argument, or /tmp/bamsort]
+ --bamsortbuflen=N                  (-M X.bam) Entries in memory sort buffer [10000]
+ --bamsortmemory=N                  (-M X.bam) Value for option -m to samtools sort [200000000]
+ --bamkeepgoodreads                 (-M X.bam) If set, do not re-map already well-mapped reads from BAM [False]
 %s
 General options:
  -?, --help                         Display this help page
@@ -125,8 +151,8 @@ Option summary (--help for all):
 Command options
  -G PREFIX file1.fa [...]           Build genome index PREFIX.stidx from fasta file(s) on command line
  -H PREFIX                          Build hash PREFIX.sthash
- -M FILE[,FILE]                     Map fastq file(s).  Use FILE.recaldata for recalibration if available
- -R FILE[,FILE]                     Compute recalibration data from fastq file(s)
+ -M FILE[,FILE]                     Map fastq/fasta/BAM file(s)
+ -A FILE                            Convert qualities; strip adapters
 
 Mapping/output options
  -g PREFIX                          Use genome index file PREFIX.stidx
@@ -167,11 +193,10 @@ default_options = """
 --lowqthreshold=10
 --seed=1
 --insertsize=250
---insertsd=60
+--insertsd=80
 --insertsize2=-2000
 --insertsd2=-1
 --maxfingerprintvariants=3
---toppostfraction=0.999
 --linearalignmentband=3
 --simulate-minindellen=0
 --simulate-maxindellen=0
@@ -181,9 +206,18 @@ default_options = """
 --gapopen=40
 --gapextend=3
 --fastaqual=30
+--maxbasequal=50
 --banding=60
 --xa-max=0
 --xa-max-discordant=0
+--padding=160
+--maxpairseeds=25
+--paircandlikethres=100
+--separator1=:
+--threads=1
+--bamsortbuflen=10000
+--bamsortmemory=200000000
+--unclipq=10
 %s
 """
 
@@ -241,12 +275,21 @@ def validate( settings ):
             raise ValueError("--insertsd must be a positive integer")
         if not -1 <= settings.maxfingerprintvariants <= 4:
             raise ValueError("--maxfingerprintvariants must be an integer between 1 and 4, or -1 for no filtering")
-        if not 0 <= settings.toppostfraction <= 1:
-            raise ValueError("--toppostfraction must be a real number between 0 and 1")
         if not 0 <= settings.substitutionrate <= 0.5:
             raise ValueError("--substitutionrate must be between 0 and 0.5")
         if not -1 <= settings.tryvariants <= 3:
             raise ValueError("--tryvariants must be -1 (autoselect), 0 (none), 1 (all) or 2, 3 (half/third of locations)")
+        if settings.keepreforder and settings.alphabetical:
+            raise ValueError("Must give at most one of (--keepreforder, --alphabetical)")
+        if len(settings.separator1) != 1:
+            raise ValueError("Separator1 must be a single character")
+        if len(settings.separator2) != 1:
+            raise ValueError("Separator2 must be a single character")
+        if settings.threads < 1:
+           raise ValueError("--threads must be a positive integer")
+        if not settings.bamsortprefix:
+           settings.bamsortprefix = [settings.output, "/tmp/bamsort"][settings.output == "-"]
+        settings.keepreforder = not settings.alphabetical
         if settings.readgroup:
             tags = ("ID","SM","LB","DS","PU","PI","CN","DT","PL")
             tagvalues = [ v.replace('#$&*',',').split(':') for v in settings.readgroup.replace(',,','#$&*').split(',') ]
@@ -298,6 +341,7 @@ def parse_options( options, settings, fail_duplicates = False ):
         if option == "-i": option = "--inputformat"
         if option == "-s": option = "--stats"
 	if option == "-o": option = "--output"
+        if option == "-t": option = "--threads"
 
         if (len(value)>0 and value[0] == "-" and option[min(2,len(option)):]+"=" not in int_opts
             and not option.startswith('--')):
@@ -332,6 +376,9 @@ def parse_options( options, settings, fail_duplicates = False ):
         elif option == "-P" or option == "--parse":
             actions.append( ("-P", value) )
 
+        elif option == "-A":
+            actions.append( ("-A", value) )
+
         elif option in ["--solexa","--solexaold","--sanger"]:
             if option[:8] == "--solexa": settings.qualitybase = "@"
             if option == "--solexaold": settings.logit = True
@@ -369,35 +416,56 @@ def build_genome( settings, logger, genomefile, arguments ):
     if len(arguments) == 0:
         raise ValueError("Please provide .fa[.gz] files to process")
     genomefile += ".stidx"
-    utils.test_write( genomefile )
+    if not settings.overwrite:
+       utils.test_write( genomefile )
     logger.log("Building genome...",1)
     logger.log("Input files: " + str( [a.split(':')[-1] for a in arguments] ), 2)
     # chop off .gz/.dz extension, if it exists
     for i in range(len(arguments)):
         if arguments[i].lower().endswith(".gz"): arguments[i] = arguments[i][:-3]
-    g = genome.genome( genomefile, parseNCBI = not settings.noparseNCBI, write=True )
+    g = genome.genome( genomefile, parseNCBI = not settings.noparseNCBI, write=True, padding = settings.padding )
     g.openFromFa( arguments, assembly=settings.assembly, species=settings.species, uri=settings.referenceuri )
 
 def build_hash( settings, logger, hashfile, arguments ):
     if len(arguments)>0: raise ValueError("Unexpected arguments on command line")
     hashfile += ".sthash"
     utils.open_genome( settings, logger )
-    utils.test_write( hashfile )
+    if not settings.overwrite:
+       utils.test_write( hashfile )
     hashtable.buildhash3( hashfile, settings, logger )
 
+def stripadapter( settings, logger, readfiles, arguments ):
+    input_filenames = readfiles.split(',') + arguments
+    if len(input_filenames) != 1:
+       raise ValueError("Please provide one .fa[.gz] file to process")
+    input_format = reader.sniff( input_filenames[0], logger )
+    if input_format != "fastq":
+       raise ValueError("Input file %s does not appear to be a fastq file" % input_filenames[0] )
+    if settings.output == "-": 
+       output = sys.stdout
+    else: 
+       if not settings.overwrite:
+          utils.test_write(settings.output)
+       output = open(settings.output,'w')
+    logger.log("Converting fastq file...",1)
+    maptools.stripadapter( input_filenames[0], output, settings, logger, filez )
+
 
 def recalibratereads( settings, logger, readfiles, arguments ):
     input_filenames = readfiles.split(',') + arguments
     if len(input_filenames) == 1: input_format = reader.sniff( input_filenames[0], logger )
     else:                         input_format = "fastq"
     recaldata_filenames = [ utils.make_recaldatafilename(settings,name) for name in input_filenames ]
-    for f in recaldata_filenames: utils.test_write( f, filetype="Recalibration data file" )
+    if not settings.overwrite:
+       for f in recaldata_filenames: 
+          if not settings.overwrite:
+             utils.test_write( f, filetype="Recalibration data file" )
     utils.open_genome( settings, logger )
 
     if input_format == "fastq":
         utils.open_hash( settings, logger )
         datareaders = [ reader.make( f, logger, settings, process_fraction=settings.recalfraction ) for f in input_filenames ]
-        mapinstance = mapper.mapper( settings.hashmap, settings.genomemap,
+        mapinstance = mapper.mapper( settings.hashmap, settings.genomemap, settings,
                                      lowqthreshold=settings.lowqthreshold,
                                      maxfingerprintvariants = settings.maxfingerprintvariants,
                                      phred_gapopen = settings.gapopen, phred_gapextend = settings.gapextend,
@@ -407,7 +475,7 @@ def recalibratereads( settings, logger, readfiles, arguments ):
 	    semapper = singleendedmapper.SingleEndMapper( settings )
 	    outputgenerator = semapper.generator( datareaders[0], mapinstance, genome )
         else:
-	    pairedreader = reader.PairedReader( datareaders[0], datareaders[1] )
+	    pairedreader = reader.PairedReader( datareaders[0], datareaders[1], logger )
 	    pemapper = singleendedmapper.PairedEndMapper( settings )
 	    outputgenerator = pemapper.generator( pairedreader, mapinstance, genome )
     else:
@@ -418,20 +486,42 @@ def recalibratereads( settings, logger, readfiles, arguments ):
     recalibrate.recalibrator( recaldata_filenames, outputgenerator, settings, maxreadlength=1000, maxq=50, pseudocount = 1 )
 
 
+
+def mapreads_background( settings, logger, queue, pipe, process_idx ):
+    """ maps reads from the queue and puts results in the pipe """
+
+    logger.setprocess( process_idx )
+    mapinstance = mapper.mapper( settings.hashmap, settings.genomemap, settings,
+                                 lowqthreshold=settings.lowqthreshold,
+                                 maxfingerprintvariants = settings.maxfingerprintvariants,
+                                 phred_gapopen = settings.gapopen, phred_gapextend = settings.gapextend,
+                                 tryvariants = settings.tryvariants,
+                                 baseentropy = settings.baseentropy)
+    readmapper = singleendedmapper.PairedEndMapper( settings )
+    datareader = reader.QueueReader( queue, readmapper, settings )
+    outputgenerator = readmapper.generator( datareader, mapinstance, settings.genomemap )
+    for data in outputgenerator:
+       update = readmapper.get_pair_stats_updates()
+       pipe.send( (data, update) )
+    pipe.send( None )
+    logger.log("Child exiting",3)
+
+
 def mapreads( settings, logger, readfiles, arguments ):
     input_filenames = readfiles.split(',') + arguments
-    if len(input_filenames) not in [1,2]: raise ValueError("Map: please provide one or two input files")
+    bams = sum( ("1234"+name.upper())[-4:] in [".BAM",".SAM"] for name in input_filenames ) == len(input_filenames)
+    if not bams and len(input_filenames) not in [1,2]: raise ValueError("Map: please provide one or two input files")
     utils.open_genome( settings, logger )
     utils.open_hash( settings, logger )
     recaldata_filenames = [ utils.make_recaldatafilename(settings,name) for name in input_filenames ]
-    mapinstance = mapper.mapper( settings.hashmap, settings.genomemap,
+    mapinstance = mapper.mapper( settings.hashmap, settings.genomemap, settings,
                                  lowqthreshold=settings.lowqthreshold,
                                  maxfingerprintvariants = settings.maxfingerprintvariants,
                                  phred_gapopen = settings.gapopen, phred_gapextend = settings.gapextend,
                                  tryvariants = settings.tryvariants,
                                  baseentropy = settings.baseentropy)
     formatinstance = formatter.make( settings, logger )
-    if len(input_filenames) == 1:
+    if len(input_filenames) == 1 or bams:
         # single-ended mapping
         if os.path.exists(recaldata_filenames[0]):
             recalhandle = open(recaldata_filenames[0],'r')
@@ -439,7 +529,10 @@ def mapreads( settings, logger, readfiles, arguments ):
             recalhandle.close()
         else:
             recaldata = [None]
-        datareader = reader.make( input_filenames[0], logger, settings, recal_data = recaldata[0], dep_data = recaldata )
+        if bams:
+           datareader = reader.make( input_filenames, logger, settings, recal_data = None, dep_data = [None] )
+        else:
+           datareader = reader.make( input_filenames[0], logger, settings, recal_data = recaldata[0], dep_data = recaldata )
     else:
         # paired-end mapping
         recaldata = [[None], [None]]
@@ -452,8 +545,9 @@ def mapreads( settings, logger, readfiles, arguments ):
 				     recal_data = recaldata[0][0], dep_data=recaldata[0] ),
                         reader.make( input_filenames[1], logger, settings, 
 				     recal_data = recaldata[1][0], dep_data=recaldata[1] ) )
-        datareader = reader.PairedReader( datareaders[0], datareaders[1] )
+        datareader = reader.PairedReader( datareaders[0], datareaders[1], logger )
     readmapper = singleendedmapper.PairedEndMapper( settings )
+    readmapper = multithread.MultiMapper( settings, readmapper, logger )
     outputgenerator = plugins.hook( 'bwa', settings, logger, readmapper, datareader, mapinstance, singleendedmapper )
     if outputgenerator == None: outputgenerator = readmapper.generator( datareader, mapinstance, settings.genomemap )
     logger.log("Mapping...",1)
@@ -470,7 +564,7 @@ def testmap( settings, logger, readfiles, arguments ):
     utils.open_genome( settings, logger )
     utils.open_hash( settings, logger )
     recaldata_filenames = [ utils.make_recaldatafilename(settings,name) for name in input_filenames ]
-    mapinstance = mapper.mapper( settings.hashmap, settings.genomemap,
+    mapinstance = mapper.mapper( settings.hashmap, settings.genomemap, settings,
                                  lowqthreshold=settings.lowqthreshold,
                                  maxfingerprintvariants = settings.maxfingerprintvariants,
                                  phred_gapopen = settings.gapopen, phred_gapextend = settings.gapextend,
@@ -498,7 +592,7 @@ def testmap( settings, logger, readfiles, arguments ):
 				     recal_data = recaldata[0][0], dep_data = recaldata[0] ),
                         reader.make( input_filenames[1], logger, settings, 
 				     recal_data = recaldata[1][0], dep_data = recaldata[1] ) )
-        pairedreader = reader.PairedReader( datareaders[0], datareaders[1] )
+        pairedreader = reader.PairedReader( datareaders[0], datareaders[1], logger )
         randomreader = reader.RandomReader( pairedreader, settings )
         themapper = singleendedmapper.PairedEndMapper( settings )
 
@@ -546,10 +640,10 @@ def simulatedata( settings, logger, readfiles, arguments ):
 				     recal_data = None, dep_data = recaldata[0] ),
                         reader.make( input_filenames[1], logger, settings, 
 				     recal_data = None, dep_data = recaldata[1] ) )
-        pairedreader = reader.PairedReader( datareaders[0], datareaders[1] )
+        pairedreader = reader.PairedReader( datareaders[0], datareaders[1], logger )
         randomreader = reader.RandomReader( pairedreader, settings )
 
-    theformatter = fastqformatter.FastqFormatter( input_filenames )
+    theformatter = fastqformatter.FastqFormatter( input_filenames, settings )
     logger.log("Simulating...",1)
     theformatter.format( randomreader )
 
@@ -594,19 +688,23 @@ def main():
         defaultopts += o
 
     defaultactions = parse_options( defaultopts, settings )
+    if not settings.separator2:
+       settings.separator2 = " "
 
     if len(settings.qualitybase)>1:
-        try:    settings.qualitybase = int( settings.qualitybase )
+        try:    settings.qualitybase = chr( int( settings.qualitybase ) )
         except: pass     # caught by validate()
 
     if settings.sensitive and settings.fast:
         raise ValueError("Can only set one of --sensitive and --fast")
+
     if settings.sensitive: 
-        settings.toppostfraction = 0.95
         settings.linearalignmentband = 4       # no effect when SSE available
         settings.maxfingerprintvariants = -1
         settings.tryvariants = 1
+        settings.maxpairseeds = max(settings.maxpairseeds, 45)
     if settings.fast: 
+        settings.paircandlikethres = 50
         settings.maxfingerprintvariants = 2
         settings.linearalignmentband = 2       # no effect when SSE available
         settings.tryvariants = 3
@@ -646,6 +744,10 @@ def main():
             build_hash( settings, logger, actiondict['-H'], arguments )
             del actiondict['-H']
 
+        elif "-A" in actiondict:
+            stripadapter( settings, logger, actiondict['-A'], arguments )
+            del actiondict['-A']
+
         elif "-R" in actiondict:
             recalibratereads( settings, logger, actiondict['-R'], arguments )
             del actiondict['-R']
